
with Ada.Unchecked_Conversion;
with Ada.Unchecked_Deallocation;

package body Allegro.Files.Memory is

    type A_Stream_Element_Array is access all Stream_Element_Array;

    procedure Free is new Ada.Unchecked_Deallocation( Stream_Element_Array, A_Stream_Element_Array );

    ----------------------------------------------------------------------------

    procedure Delete( memInfo : in out A_Memory_Info ) is
        procedure Free is new Ada.Unchecked_Deallocation( Memory_Info, A_Memory_Info );
    begin
        if memInfo /= null then
            if memInfo.consumed then
                Free( A_Stream_Element_Array(memInfo.mem) );
            end if;
            Free( memInfo );
        end if;
    end Delete;

    ----------------------------------------------------------------------------

    function To_Memory_Info is new Ada.Unchecked_Conversion( Address, A_Memory_Info );

    procedure Mem_Move( dst, src : Address; len : Integer );
    pragma Import( C, Mem_Move, "memmove" );

    ---------------------------------------------------------------------------

    function Mem_pf_close( userdata : Address ) return Integer is
        memInfo : A_Memory_Info := To_Memory_Info( userdata );
    begin
        Delete( memInfo );
        return 0;
    end Mem_pf_close;

    ---------------------------------------------------------------------------

    function Mem_pf_getc( userdata : Address ) return Integer is
        memInfo : constant A_Memory_Info := To_Memory_Info( userdata );
    begin
        if memInfo.offset > memInfo.mem'Last then
            return -1;
        else
            memInfo.offset := memInfo.offset + 1;
            return Integer(memInfo.mem(memInfo.offset - 1));
        end if;
    end Mem_pf_getc;

    ---------------------------------------------------------------------------

    function Mem_pf_ungetc( c : Integer; userdata : Address ) return Integer is
        memInfo : constant A_Memory_Info := To_Memory_Info( userdata );
    begin
        if memInfo.offset > memInfo.mem'First then
            return c;
        else
            return -1;
        end if;
    end Mem_pf_ungetc;

    ---------------------------------------------------------------------------

    function Mem_pf_fread( p        : Address;
                           n        : Interfaces.C.long;
                           userdata : Address ) return Interfaces.C.long is
        memInfo : constant A_Memory_Info := To_Memory_Info( userdata );
        actual  : Integer := 0;
    begin
        if memInfo.offset <= memInfo.mem'Last then
            actual := Integer'Min( Integer(n), Integer(memInfo.mem'Last - memInfo.offset) + 1 );
            Mem_Move( p, memInfo.mem(memInfo.offset)'Address, actual );
            memInfo.offset := memInfo.offset + Stream_Element_Offset(actual);
        end if;
        return Interfaces.C.long(actual);
    end Mem_pf_fread;

    ---------------------------------------------------------------------------

    function Mem_pf_putc( c : Integer; userdata : Address ) return Integer is
        pragma Unreferenced( c, userdata );
    begin
        return -1;
    end Mem_pf_putc;

    ---------------------------------------------------------------------------

    function Mem_pf_fwrite( p        : Address;
                            n        : Interfaces.C.long;
                            userdata : Address ) return Interfaces.C.long is
        pragma Unreferenced( p, n, userdata );
    begin
        return 0;
    end Mem_pf_fwrite;

    ---------------------------------------------------------------------------

    function Mem_pf_seek( userdata : Address; offset : Integer ) return Integer is
        memInfo : constant A_Memory_Info := To_Memory_Info( userdata );
        actual  : Integer;
    begin
        actual := Integer'Min( offset, Integer(memInfo.mem'Last - memInfo.offset) + 1 );
        memInfo.offset := memInfo.offset + Stream_Element_Offset(actual);
        if offset = actual then
            return 0;
        else
            return -1;
        end if;
    end Mem_pf_seek;

    ---------------------------------------------------------------------------

    function Mem_pf_feof( userdata : Address ) return Integer is
        memInfo : constant A_Memory_Info := To_Memory_Info( userdata );
    begin
        if memInfo.offset > memInfo.mem'Last then
            return 1;
        else
            return 0;
        end if;
    end Mem_pf_feof;

    ---------------------------------------------------------------------------

    function Mem_pf_ferror( userdata : Address ) return Integer is
        pragma Unreferenced( userdata );
    begin
        return 0;
    end Mem_pf_ferror;

    ---------------------------------------------------------------------------

    mem_vtable : constant A_Packfile_Vtable :=
        new Packfile_Vtable'(
            Mem_pf_close'Access,
            Mem_pf_getc'Access,
            Mem_pf_ungetc'Access,
            Mem_pf_fread'Access,
            Mem_pf_putc'Access,
            Mem_pf_fwrite'Access,
            Mem_pf_seek'Access,
            Mem_pf_feof'Access,
            Mem_pf_ferror'Access
        );

    ---------------------------------------------------------------------------

    function Pack_FOpen( mem     : access Stream_Element_Array;
                         consume : Boolean := False ) return A_Packfile is
        memInfo : constant A_Memory_Info := new Memory_Info'(mem, mem'First, consume);
    begin
        return Pack_FOpen_Vtable( mem_vtable, memInfo.all'Address );
    end Pack_FOpen;

end Allegro.Files.Memory;
