
with Ada.Characters.Handling;           use Ada.Characters.Handling;
with Ada.Command_Line;                  use Ada.Command_Line;
with Ada.Text_IO;
with Ada.Strings.Unbounded;             use Ada.Strings.Unbounded;
with Applications;
with Allegro.Bitmaps;                   use Allegro.Bitmaps;
with Allegro.Image_Files;               use Allegro.Image_Files;
with GNAT.Directory_Operations.Iteration;
 use GNAT.Directory_Operations.Iteration;
with GNAT.OS_Lib;                       use GNAT.OS_Lib;
with Support;                           use Support;
with Support.Paths;                     use Support.Paths;
with Version;

pragma Elaborate_All( Applications );

package body Applications.Bmpconvert is

    function Create_Application return A_Application is
        this : constant A_Application := new Bmpconvert_Application;
    begin
        this.Construct;
        return this;
    end Create_Application;

    ----------------------------------------------------------------------------

    overriding
    function Get_Name( this : access Bmpconvert_Application ) return String is
        pragma Unreferenced( this );
    begin
        return "Bmpconvert";
    end Get_Name;

    ----------------------------------------------------------------------------

    overriding
    function Get_Window_Title( this : access Bmpconvert_Application ) return String is
        pragma Unreferenced( this );
    begin
        return "bmpconvert";
    end Get_Window_Title;

    ----------------------------------------------------------------------------

    overriding
    function Init( this : access Bmpconvert_Application ) return Boolean is
    begin
        return this.Init( 128, 64, app_scale => 1, app_windowed => True );
    end Init;

    ----------------------------------------------------------------------------

    overriding
    procedure Run( this       : access Bmpconvert_Application;
                   returnCode : in out Integer ) is
        pragma Unreferenced( this );

        -----------------------------------------------------------------------

        procedure Display_Usage is
        begin
            Ada.Text_IO.Put_Line( "Converts a directory of 256 color BMP files to 24 bits" );
            Ada.Text_IO.New_Line;
            Ada.Text_IO.Put_Line( "Usage:" );
            Ada.Text_IO.Put_Line( "  bmpconvert <directory>" );
            Ada.Text_IO.New_Line;
            returnCode := 1;
        end Display_Usage;

        ------------------------------------------------------------------------

        procedure Convert( item : String; index : Positive; quit : in out Boolean ) is
            pragma Unreferenced( index, quit );
            filename : constant String := Normalize_Pathname( item );
            bmp      : A_Bitmap;
        begin
            Ada.Text_IO.Put( "  Converting " & filename );
            bmp := Load_BMP( filename & ASCII.NUL, null );
            if bmp /= null then
                if Save_BMP( To_Lower( filename ), bmp, null ) then
                    Ada.Text_IO.New_Line;
                else
                    Ada.Text_IO.Put_Line( ": Write error" );
                end if;
                Destroy_Bitmap( bmp );
            else
                Ada.Text_IO.Put_Line( ": Read error" );
            end if;
        end Convert;

        ------------------------------------------------------------------------

        procedure Convert_All is new Wildcard_Iterator( Convert );

        dir : Unbounded_String := To_Unbounded_String( "." );
    begin
        Ada.Text_IO.New_Line;
        Ada.Text_IO.Put_Line( "BMPConvert v" & Version.Release & " - Kevin Wellwood " &
                              "Copyright (C) " & Version.Copyright_Year );
        Ada.Text_IO.New_Line;

        if Argument_Count > 1 then
            Display_Usage;
            return;
        elsif Argument_Count = 1 then
            dir := To_Unbounded_String( Argument( 1 ) );
        end if;

        if not GNAT.OS_Lib.Is_Directory( To_String( dir ) ) then
            Ada.Text_IO.Put_Line( "Unable to find " & Normalize_Pathname( To_String( dir ) ) );
            return;
        end if;

        Set_Color_Conversion( 16#04FFFFFF# );

        Ada.Text_IO.Put_Line( "Converting " & Normalize_Pathname( To_String( dir ) ) & Slash & "*.bmp" );
        dir := To_Unbounded_String( Normalize_Pathname( To_String( dir ) ) & Slash & "*.[bB][mM][pP]" );
        Convert_All( To_String( dir ) );
    end Run;

    ----------------------------------------------------------------------------

begin

    Register_Allocator( Create_Application'Access );

end Applications.Bmpconvert;
