
with Allegro;                           use Allegro;
with Allegro.Graphics;                  use Allegro.Graphics;
with Debugging;                         use Debugging;
with Engine_Debugging;                  use Engine_Debugging;
with Events.Application;                use Events.Application;

package body Game_Views.Window_Hooks is

    close_window : Boolean := False;
    pragma Atomic( close_window );
    pragma Volatile( close_window );

    ----------------------------------------------------------------------------

    procedure Close_Window_Callback is
    begin
        close_window := True;
    end Close_Window_Callback;

    ----------------------------------------------------------------------------

    procedure Switch_In_Callback is
    begin
        Queue_App_Focus;
    end Switch_In_Callback;

    ----------------------------------------------------------------------------

    procedure Switch_Out_Callback is
    begin
        Queue_App_Blur;
    end Switch_Out_Callback;

    ----------------------------------------------------------------------------

    procedure Initialize is
    begin
        if Set_Close_Button_Callback( Close_Window_Callback'Access ) /= 0 then
            pragma Debug( Dbg( "Game_Views.Window_Hooks.Initialize Warning: " &
                               "Set_Close_Button_Callback: " & Allegro_Error,
                               D_INIT, WARNING ) );
            null;
        end if;

        if Set_Display_Switch_Callback( SWITCH_IN, Switch_In_Callback'Access ) /= 0 then
            pragma Debug( Dbg( "Game_Views.Window_Hooks.Initialize Warning: " &
                               "SWITCH_IN callback unsupported.",
                               D_INIT, WARNING ) );
            null;
        end if;

        if Set_Display_Switch_Callback( SWITCH_OUT, Switch_Out_Callback'Access ) /= 0 then
            pragma Debug( Dbg( "Game_Views.Window_Hooks.Initialize Warning: " &
                               "SWITCH_OUT callback is unsupported.",
                               D_INIT, WARNING ) );
            null;
        end if;
    end Initialize;

    ----------------------------------------------------------------------------

    procedure Finalize is
    begin
        if Set_Close_Button_Callback( null ) /= 0 then
            null;
        end if;
        Remove_Display_Switch_Callback( Switch_In_Callback'Access );
        Remove_Display_Switch_Callback( Switch_Out_Callback'Access );
    end Finalize;

    ----------------------------------------------------------------------------

    function Close_Window_Clicked return Boolean is
        result : Boolean := False;
    begin
        if close_window then
            close_window := False;
            result := True;
        end if;
        return result;
    end Close_Window_Clicked;

end Game_Views.Window_Hooks;
