
with Ada.Unchecked_Deallocation;
with GNAT.OS_Lib;                       use GNAT.OS_Lib;

package body Support.OSX is

    function Executable_Extension return String is
    begin
        return "";
    end Executable_Extension;

    ----------------------------------------------------------------------------

    function Executable_Path return String is

        type PString is new String;

        subtype OSErr is Short_Integer;

        subtype OSStatus is Integer;

        type OSType is new Unsigned_32;

        type SInt8 is range - 128 .. 127;
            for SInt8'Size use 8;
        type Ptr is access SInt8;

        type FSRef is record
            hidden : String(1..80);
        end record;

        type FSSpec is record
            vRefNum : Short_Integer;
            parID   : Integer;
            name    : PString(1..256);
        end record;
        pragma Pack( FSSpec );
        type FSSpecPtr is access all FSSpec;

        type ProcessSerialNumber is new Unsigned_64;

        type ProcessInfoRec is record
            processInfoLength : Unsigned_32 := ProcessInfoRec'Size/8;
            processName       : Unsigned_32 := 0;    -- StringPtr
            processNumber     : ProcessSerialNumber;
            processType       : Unsigned_32;
            processSignature  : OSType;
            processMode       : Unsigned_32;
            processLocation   : Ptr;
            processSize       : Unsigned_32;
            processFreeMem    : Unsigned_32;
            processLauncher   : ProcessSerialNumber;
            processLaunchDate : Unsigned_32;
            processActiveTime : Unsigned_32;
            processAppSpec    : FSSpecPtr := null;
        end record;
        pragma Pack( ProcessInfoRec );

        procedure Free is new Ada.Unchecked_Deallocation( FSSpec, FSSpecPtr );

        function FSpMakeFSRef( source : access FSSpec;
                               newRef : access FSRef ) return OSErr;
        pragma Import( C, FSpMakeFSRef, "FSpMakeFSRef" );

        function FSRefMakePath( ref         : access FSRef;
                                path        : String;
                                maxPathSize : Unsigned_32 ) return OSStatus;
        pragma Import( C, FSRefMakePath, "FSRefMakePath" );

        function GetCurrentProcess( PSN : access ProcessSerialNumber ) return OSErr;
        pragma Import( C, GetCurrentProcess, "GetCurrentProcess" );

        function GetProcessInformation( PSN  : access ProcessSerialNumber;
                                        info : access ProcessInfoRec ) return OSErr;
        pragma Import( C, GetProcessInformation, "GetProcessInformation" );

        ------------------------------------------------------------------------

        info : aliased ProcessInfoRec;
        psn  : aliased ProcessSerialNumber;
        ref  : aliased FSRef;
        path : aliased String(1..512) := (others=>ASCII.NUL);
        pragma Warnings( Off, path );
    begin
        if GetCurrentProcess( psn'Access ) = 0 then
            info.processInfoLength := ProcessInfoRec'Size/8;
            info.processAppSpec    := new FSSpec;
            if GetProcessInformation( psn'Access, info'Access ) = 0 then
                if FSpMakeFSRef( info.processAppSpec, ref'Access ) = 0 then
                    Free( info.processAppSpec );
                    if FSRefMakePath( ref'Access, path, path'Length ) = 0 then
                        for i in path'Range loop
                            if path(i) = ASCII.NUL then
                                return path(1..i-1);
                            end if;
                        end loop;
                        return path;
                    end if;
                end if;
            end if;
            Free( info.processAppSpec );
        end if;
        return ".";
    exception
        when others =>
            return ".";
    end Executable_Path;

    ----------------------------------------------------------------------------

    function Home_Directory return String is
    begin
        return Normalize_Pathname( "~/" );
    end Home_Directory;

    ----------------------------------------------------------------------------

    function System_Font_Directory return String is
    begin
        return "/System/Library/Fonts/";
    end System_Font_Directory;

end Support.OSX;
