
with Ada.Characters.Handling;           use Ada.Characters.Handling;
with Allegro.Bitmaps.Direct_Access;     use Allegro.Bitmaps.Direct_Access;
with Allegro.Graphics;                  use Allegro.Graphics;
with Allegro.Files.Memory;              use Allegro.Files.Memory;
with Allegro.Image_Files;               use Allegro.Image_Files;
with Allegro.Truecolor;                 use Allegro.Truecolor;
--with Debugging;                         use Debugging;
with LoadPNG;                           use LoadPNG;

package body Resources.Images is

    function Load_BMP( data : not null access Stream_Element_Array ) return A_Bitmap is
        packfile  : A_Packfile;
        bmp       : A_Bitmap;
        col       : Color_Type;
        magicPink : constant Color_Type := Makecol( 255, 0, 255 );
    begin
        packfile := Pack_FOpen( data );
        bmp := Load_BMP_pf( packfile, null );

        -- Give all pixels except magic pink an opaque alpha value. bitmaps that
        -- are read with Load_BMP have their entire alpha layer set to 0 and we
        -- need to make it solid for drawing later.
        if Get_Color_Depth >= 24 then
            for y in 0..Get_Height( bmp ) - 1 loop
                for x in 0..Get_Width( bmp ) - 1 loop
                    col := Memory_Getpixel_32( bmp, x, y );
                    if col /= magicPink then
                        -- If the pixel isn't magic pink then set the alpha
                        -- channel to 100% opaque.
                        Memory_Putpixel_32( bmp, x, y, Makeacol( GetR32( col ),
                                                                 GetG32( col ),
                                                                 GetB32( col ),
                                                                 255 ) );
                    end if;
                end loop;
            end loop;
        end if;

        Pack_FClose0( packfile );
        return bmp;
    exception
        when others =>
            Pack_FClose0( packfile );
            raise;
    end Load_BMP;

    ----------------------------------------------------------------------------

    function Load_PNG( data : not null access Stream_Element_Array ) return A_Bitmap is
        bmp : A_Bitmap;
    begin
        bmp := Load_Memory_PNG( data.all'Address, data'Length, null );
        return bmp;
    end Load_PNG;

    --==========================================================================

    function Is_Format_Supported( format : String ) return Boolean is
        lower : constant String := To_Lower( format );
    begin
        return lower = "bmp" or else
               lower = "png";
    end Is_Format_Supported;

    ----------------------------------------------------------------------------

    function Load_From_Memory( format : String;
                               data   : not null access Stream_Element_Array ) return A_Bitmap is
        lower : constant String := To_Lower( format );
    begin
        if lower = "bmp" then
            return Load_BMP( data );
        elsif lower = "png" then
            return Load_PNG( data );
        else
            raise RESOURCE_FORMAT_ERROR with "Unrecognized image format '" & lower & "'";
        end if;
    end Load_From_Memory;

end Resources.Images;
