
with Ada.Strings.Unbounded;             use Ada.Strings.Unbounded;
with Interfaces;                        use Interfaces;

package Support is

    function "*"( i : Integer; f : Float ) return Float;

    function "*"( f : Float; i : Integer ) return Float;

    function "/"( i : Integer; f : Float ) return Float;

    function "/"( f : Float; i : Integer ) return Float;

    function "-"( f : Float; i : Integer ) return Float;

    function "-"( i : Integer; f : Float ) return Float;

    function "+"( f : Float; i : Integer ) return Float;

    function "+"( i : Integer; f : Float ) return Float;

    function "**"( l, r : Long_Float ) return Long_Float;

    function Constrain( val, min, max : Float ) return Float;

    function Constrain( val, min, max : Integer ) return Integer;

    function Div_Ceil( a, b : Integer ) return Integer;

    function Floor( x : Float ) return Integer;

    function Max( a, b : Integer ) return Integer;

    function Max( a, b : Float ) return Float;

    function Min( a, b : Integer ) return Integer;

    function Min( a, b : Float ) return Float;

    ----------------------------------------------------------------------------

    function "&"( l : String; r : Unbounded_String ) return String;

    function "&"( l : Unbounded_String; r : String ) return String;

    -- Capitalizes the words in the string using a set of delimiters.
    function Capitalize( str : String ) return String;
    pragma Postcondition( Capitalize'Result'Length = str'Length );

    function Case_Eq( l, r : String ) return Boolean;

    function Case_Eq( l, r : Unbounded_String ) return Boolean;

    function Case_Eq( l : Unbounded_String; r : String ) return Boolean;

    function Case_Eq( l : String; r : Unbounded_String ) return Boolean;

    -- Returns True if 'str' ends with 'ending'. Comparison is case sensitive.
    function Ends_With( str : String; ending : String ) return Boolean;

    -- Iterate over words in a string separated by whitespace.
    procedure Iterate_Words( phrase  : String;
                             examine : access procedure( word : String ) );

    ----------------------------------------------------------------------------

    -- Returns -1 if the file does not exist.
    function File_Length( path : String ) return Long_Integer;
    pragma Postcondition( File_Length'Result >= -1 );

    ----------------------------------------------------------------------------

    -- Extracts a readable Ada unit name from a source reference line. If the
    -- source reference doesn't contain symbols then an empty string will be
    -- returned because the unit name can't be determined.
    function Source_Ref_To_Unit_Name( ref : String ) return String;

    ----------------------------------------------------------------------------

    -- Returns True if rectangle A contains point B.
    function Contains( ax1, ay1, ax2, ay2, bx, by : Integer ) return Boolean;

    -- Returns the value of 'point' as it would be snapped to a grid of size
    -- 'gridSize'. If 'centered' is True, the snap will occur in the middle of
    -- the grid lines instead of on them.
    function Grid_Snap( point    : Integer;
                        gridSize : Positive;
                        centered : Boolean := False ) return Integer;

    -- Returns True if rectangles A and B intersect.
    function Intersect( ax1, ay1, ax2, ay2, bx1, by1, bx2, by2 : Integer ) return Boolean;

    ----------------------------------------------------------------------------

    function Image( i : Integer ) return String;
    pragma Postcondition( Image'Result'Length > 0 );

    function Image( u : Unsigned_32 ) return String;
    pragma Postcondition( Image'Result'Length > 0 );

    -- Returns a string image of a floating point number, where 'precision' is
    -- the number of places after the decimal to render.
    function Image( f : Float; precision : Natural := 3 ) return String;
    pragma Postcondition( Image'Result'Length > 0 );

private

    package OS is

        -- Returns the file extension for executable files, not including a dot.
        function Executable_Extension return String;

        -- Returns the absolute path of the current executable file.
        function Executable_Path return String;

        -- Returns the home directory.
        function Home_Directory return String;

        -- Returns the directory for system fonts on the local machine.
        function System_Font_Directory return String;

    end OS;

    pragma Import( C, "**", "pow" );

end Support;
