
--with Debugging;                         use Debugging;

package body Widgets.Buttons.Groups is

    function Create_Button_Group return A_Button_Group is
        this : A_Button_Group := new Button_Group;
    begin
        this.Construct;
        return this;
    exception
        when others =>
            Delete( this );
            raise;
    end Create_Button_Group;

    ----------------------------------------------------------------------------

    not overriding
    procedure Add( this : access Button_Group; button : not null A_Button ) is
    begin
        if button.Get_State then
            if this.pressed /= null then
                this.pressed.Set_State( False );
            end if;
            this.pressed := button;
        end if;
        button.Add_Listener( A_Button_Listener(this) );
        this.buttons.Append( button );
    end Add;

    ----------------------------------------------------------------------------

    not overriding
    procedure Add_Listener( this     : access Button_Group;
                            listener : not null A_Button_Group_Listener ) is
    begin
        this.listeners.Append( listener );
    end Add_Listener;

    ----------------------------------------------------------------------------

    not overriding
    procedure Clear( this : access Button_Group ) is
        crsr : Button_Collection.Cursor := this.buttons.First;
    begin
        while Has_Element( crsr ) loop
            Element( crsr ).Remove_Listener( A_Button_Listener(this) );
            crsr := Next( crsr );
        end loop;
        this.buttons.Clear;
        this.pressed := null;
    end Clear;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Button_Group ) is
    begin
        this.Clear;
        Object(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    not overriding
    procedure Dispatch_Action( this   : access Button_Group;
                               id     : Action_Id;
                               source : not null A_Widget ) is
        action : A_Button_Group_Action;

        procedure Examine( pos : Action_Listeners.Cursor ) is
        begin
            Action_Listeners.Element( pos ).all( action );
        end Examine;

    begin
        action := new Button_Group_Action;
        action.Construct( id, source );
        this.listeners.Iterate( Examine'Unrestricted_Access );
        Delete( action );
    end Dispatch_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this   : access Button_Group;
                             action : A_Button_Action ) is
        prevPressed : A_Button;
    begin
        if action.Get_Id = Press then
            if action.Get_Source /= A_Widget(this.pressed) then
                prevPressed := this.pressed;
                this.pressed := A_Button(action.Get_Source);
                if prevPressed /= null then
                    if prevPressed.Get_State then
                        prevPressed.Set_State( False );
                    else
                        -- group is set again; a button other than the most
                        -- recently pressed one has been pressed
                        this.Dispatch_Action( Group_Set, A_Widget(action.Get_Source) );
                    end if;
                else
                    -- group is set for the first time
                    this.Dispatch_Action( Group_Set, A_Widget(action.Get_Source) );
                end if;
            else
                -- group is set again; the button that was most recently pressed
                -- was pressed again
                this.Dispatch_Action( Group_Set, A_Widget(action.Get_Source) );
            end if;
        elsif action.Get_Id = Release then
            if action.Get_Source = A_Widget(this.pressed) then
                if this.keep_selected then
                    this.pressed.Set_State( True );
                else
                    this.Dispatch_Action( Group_Cleared, A_Widget(action.Get_Source) );
                end if;
            end if;
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    not overriding
    procedure Remove_Listener( this     : access Button_Group;
                               listener : not null A_Button_Group_Listener ) is
        crsr : Action_Listeners.Cursor := this.listeners.Find( listener );
    begin
        if Action_Listeners.Has_Element( crsr ) then
            this.listeners.Delete( crsr );
        end if;
    end Remove_Listener;

    ----------------------------------------------------------------------------

    not overriding
    procedure Set_Keep_Selected( this : access Button_Group; keep : Boolean ) is
    begin
        this.keep_selected := keep;
    end Set_Keep_Selected;

    ----------------------------------------------------------------------------

    overriding
    function To_String( this : access Button_Group ) return String is

        function Get_Button_Names return String is
            crsr  : Button_Collection.Cursor := this.buttons.First;
            names : Unbounded_String;
        begin
            while Has_Element( crsr ) loop
                if crsr = this.buttons.First then
                    names := names & "'" & Element( crsr ).Get_Text & "'";
                else
                    names := names & ", '" & Element( crsr ).Get_Text & "'";
                end if;
                crsr := Next( crsr );
            end loop;
            return To_String( names );
        end Get_Button_Names;

    begin
        if not this.buttons.Is_Empty then
            return "<" & this.Get_Class_Name & " : " & Get_Button_Names & ">";
        else
            return "<" & this.Get_Class_Name & " : [Empty]>";
        end if;
    end To_String;

    ----------------------------------------------------------------------------

    not overriding
    procedure Unset( this  : access Button_Group;
                     force : Boolean := False ) is
        keep_selected : constant Boolean := this.keep_selected;
    begin
        if this.pressed /= null then
            if force then
                this.keep_selected := False;
            end if;
            this.pressed.Set_State( False );
            this.pressed := null;
            this.keep_selected := keep_selected;
        end if;
    end Unset;

    ----------------------------------------------------------------------------

    procedure Delete( this : in out A_Button_Group ) is
    begin
        Delete( A_Object(this) );
    end Delete;

    --==========================================================================

    procedure Delete( this : in out A_Button_Group_Action ) is
    begin
        Delete( A_Object(this) );
    end Delete;

end Widgets.Buttons.Groups;
