
with Ada.Unchecked_Conversion;
with Debugging;                         use Debugging;
with GNAT.OS_Lib;                       use GNAT.OS_Lib;
with Interfaces.C;                      use Interfaces.C;
with System;                            use System;
--with Win32;                             use Win32;
--with Win32.Winbase;                     use Win32.Winbase;
--with Win32.Windef;                      use Win32.Windef;

package body Support.Win is

    function Executable_Extension return String is
    begin
        return "exe";
    end Executable_Extension;

    ----------------------------------------------------------------------------

    function Executable_Path return String is

        -- Begin win32ada library workaround
        --
        MAX_PATH : constant := 260;

        subtype PVOID     is System.Address;
        subtype HANDLE    is PVOID;
        subtype HINSTANCE is HANDLE;
        subtype CHAR      is Interfaces.C.char;
        type    PCHAR     is access all CHAR;
        subtype LPSTR     is PCHAR;
        subtype ULONG     is Interfaces.C.unsigned_long;
        subtype DWORD     is ULONG;

        function GetModuleFileNameA( hModule    : HINSTANCE;
                                     lpFilename : LPSTR;
                                     nSize      : DWORD
                                   ) return DWORD;
        pragma Import (Stdcall, GetModuleFileNameA, "GetModuleFileNameA");
        --
        -- End win32ada library workaround

        function To_LPSTR is new Ada.Unchecked_Conversion( System.Address, LPSTR );

        path : aliased String(1..MAX_PATH);
        len  : Interfaces.C.unsigned_long;
    begin
        len := GetModuleFileNameA( Null_Address, To_LPSTR( path(path'First)'Address ), MAX_PATH );
        if len = 0 then
            -- error getting executable path
            return ".";
        end if;
        return path(1..Integer(len));
    end Executable_Path;

    ----------------------------------------------------------------------------

    function Home_Directory return String is
        aStr : GNAT.OS_Lib.String_Access := Getenv( "USERPROFILE" );
        str  : constant String := String'(aStr.all);
    begin
        Free( aStr );
        return str & "\";
    end Home_Directory;

    ----------------------------------------------------------------------------

    function System_Font_Directory return String is
        str : GNAT.OS_Lib.String_Access := Getenv( "SystemRoot" );
    begin
        if str /= null then
            declare
                result : constant String := str.all;
            begin
                Free( str );
                return result & "\fonts\";
            end;
        else
            Dbg( "Warning: Environment variable SystemRoot not found" );
        end if;
        return "";
    end System_Font_Directory;

end Support.Win;
