
with Ada.Command_Line;                  use Ada.Command_Line;
with Ada.Strings.Unbounded;             use Ada.Strings.Unbounded;
with Ada.Text_IO;
with Applications;
with Preferences;
with Support.Paths;                     use Support.Paths;
with Tiles.Libraries;                   use Tiles.Libraries;
with Tiles.Libraries.Packing;           use Tiles.Libraries.Packing;
with Version;

pragma Elaborate_All( Applications );
pragma Elaborate_All( Preferences );
pragma Elaborate_All( Support.Paths );

package body Applications.Gfxpack is

    function Create_Application return A_Application is
        this : constant A_Application := new Gfxpack_Application;
    begin
        this.Construct;
        return this;
    end Create_Application;

    ----------------------------------------------------------------------------

    overriding
    function Get_Name( this : access Gfxpack_Application ) return String is
        pragma Unreferenced( this );
    begin
        return "GfxPack";
    end Get_Name;

    ----------------------------------------------------------------------------

    overriding
    function Get_Window_Title( this : access Gfxpack_Application ) return String is
        pragma Unreferenced( this );
    begin
        return "GfxPack";
    end Get_Window_Title;

    ----------------------------------------------------------------------------

    overriding
    function Init( this : access Gfxpack_Application ) return Boolean is
    begin
        return this.Init( 128, 64, app_scale => 1, app_windowed => True );
    end Init;

    ----------------------------------------------------------------------------

    overriding
    procedure Run( this : access Gfxpack_Application; returnCode : in out Integer ) is
        pragma Unreferenced( this );
        quiet : Boolean := False;

        ------------------------------------------------------------------------

        procedure Output_Line( str : String ) is
        begin
            if not quiet then
                Ada.Text_IO.Put_Line( str );
            end if;
        end Output_Line;

        ------------------------------------------------------------------------

        procedure Display_Title is
        begin
            Output_Line( "" );
            Output_Line( "GfxPack v" & Version.Release & " - Kevin Wellwood " &
                         "Copyright (C) " & Version.Copyright_Year );
            Output_Line( "" );
        end Display_Title;

        ------------------------------------------------------------------------

        procedure Display_Usage is
        begin
            Display_Title;
            Output_Line( "Usage:" );
            Output_Line( "  gfxpack [-c] [-q] <library_name>" );
            Output_Line( "" );
            Output_Line( "  -c : Compress archive" );
            Output_Line( "  -q : Quiet mode" );
            Output_Line( "" );
            returnCode := 1;
        end Display_Usage;

        ------------------------------------------------------------------------

        name     : Unbounded_String;
        compress : Boolean := False;
    begin
        if Argument_Count < 1 or else Argument_Count > 3 then
            Display_Usage;
            return;
        end if;

        for i in 1..Argument_Count - 1 loop
            if Argument( i ) = "-c" then
                compress := True;
            elsif Argument( i ) = "-q" then
                quiet := True;
            else
                Display_Usage;
                return;
            end if;
        end loop;

        name := To_Unbounded_String( Argument( Argument_Count ) );
        if Index( name, "-" ) = 1 then
            Display_Usage;
            return;
        end if;

        Display_Title;

        -- create the library from individual resources
        if not Pack_Library( To_String( name ), compress, Output_Line'Unrestricted_Access ) then
            returnCode := 1;
        end if;
    end Run;

    ----------------------------------------------------------------------------

begin

    Register_Allocator( Create_Application'Access );

    Preferences.Set_Default( "application", "media", "media" & Slash );

end Applications.Gfxpack;
