
with Allegro.Keyboard;                  use Allegro.Keyboard;
with Applications;                      use Applications;
--with Debugging;                         use Debugging;
with Events.Entities;                   use Events.Entities;
with Game_Views;                        use Game_Views;
with Support;                           use Support;
with Values;                            use Values;
with Widgets.Buttons.Pushes;            use Widgets.Buttons.Pushes;
with Widgets.Containers.Windows;        use Widgets.Containers.Windows;
with Widgets.Input_Boxes;               use Widgets.Input_Boxes;
with Widgets.Labels;                    use Widgets.Labels;

package body Widgets.Containers.Panels.Dialogs.Entity_Attributes is

    VALUES_WIDTH : constant := 100;   -- width of the value column
    SPACING      : constant := 3;

    ----------------------------------------------------------------------------

    function Create_Entity_Attributes_Dialog( view       : not null access Game_Views.Game_View'Class;
                                              id         : String;
                                              eid        : Entity_Id;
                                              class      : String;
                                              attributes : not null A_Association ) return A_Dialog is
        this : A_Dialog := new Entity_Attributes_Dialog;
    begin
        Entity_Attributes_Dialog(this.all).Construct( view, id, eid, class, attributes );
        return this;
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Entity_Attributes_Dialog;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this       : access Entity_Attributes_Dialog;
                         view       : not null access Game_Views.Game_View'Class;
                         id         : String;
                         eid        : Entity_Id;
                         class      : String;
                         attributes : not null A_Association ) is
        labels      : Widget_Lists.List;
        inputs      : Widget_Lists.List;
        nextTop     : Integer := 0;
        namesWidth  : Integer := 0;
        totalHeight : Integer := 0;

        ------------------------------------------------------------------------

        procedure Add_Attribute( name : String; value : A_Value ) is
            val   : constant String := value.As_String;
            label : A_Label;
            input : A_Input_Box;
        begin
            label := Create_Label( this.Get_View, To_String( this.id & ":lbl" & name ), name & ":" );
            label.Set_Align( Align_Right );
            labels.Append( A_Widget(label) );
            if A_Widget(label).Get_Min_Width > namesWidth then
                namesWidth := A_Widget(label).Get_Min_Width;
            end if;
            totalHeight := totalHeight + SPACING + A_Widget(label).Get_Min_Height;
            this.Add( A_Widget(label) );

            input := Create_Input_Box( this.Get_View, To_String( this.id & ":ib" & name ) );
            input.Set_Text( val );
            inputs.Append( A_Widget(input) );
            this.Add( A_Widget(input) );
        end Add_Attribute;

        ------------------------------------------------------------------------

        procedure Layout_Attributes is
            lcrsr     : Widget_Lists.Cursor := labels.First;
            icrsr     : Widget_Lists.Cursor := inputs.First;
            label     : A_Widget;
            input     : A_Widget;
            rowHeight : Integer;
        begin
            while Widget_Lists.Has_Element( lcrsr ) loop
                label := Widget_Lists.Element( lcrsr );
                rowHeight := label.Get_Min_Height;
                label.Set_Layout_LTWH( SPACING, nextTop + SPACING, namesWidth, rowHeight );
                lcrsr := Widget_Lists.Next( lcrsr );

                input := Widget_Lists.Element( icrsr );
                input.Set_Layout_LTRB( SPACING + namesWidth + SPACING,
                                       nextTop + SPACING,
                                       SPACING, -(nextTop + SPACING + rowHeight) );
                icrsr := Widget_Lists.Next( icrsr );
                nextTop := nextTop + SPACING + rowHeight;
            end loop;
        end Layout_Attributes;

        ------------------------------------------------------------------------

        function Dialog_Icon( class : String ) return String is
        begin
            if class = "trigger" then
                return "trigger_icon";
            else
                return "";
            end if;
        end Dialog_Icon;

        ------------------------------------------------------------------------

        theme  : constant A_Theme := Get_Application.Get_Theme;
        button : A_Button;
    begin
        Dialog(this.all).Construct( view, id,
                                    Capitalize( class ) & " Attributes",
                                    Dialog_Icon( class ) );
        this.eAttributes := Copy( attributes );
        this.eid := eid;

        attributes.Iterate( Add_Attribute'Access );
        Layout_Attributes;

        labels.Clear;
        inputs.Clear;

        this.Set_Layout_Center( theme.Border_Width * 2 +
                                Integer'Max( SPACING + namesWidth + SPACING + VALUES_WIDTH + SPACING,
                                             this.titlePadding + theme.Label_Width( theme.Get_Bitmap( this.icon ),
                                                                                    this.Get_Title,
                                                                                    To_String( this.fontName ),
                                                                                    this.fontSize ) + this.titlePadding ),
                                totalHeight + this.Get_Min_Height + 30 );

        button := Create_Push_Button( view, this.id & ":btnApply", "Apply" );
        button.Set_Layout_LTWH( -65, -5, 55, 20 );
        button.Add_Listener( A_Button_Listener(this) );
        this.Add( A_Widget(button) );

        button := Create_Push_Button( view, this.id & ":btnCancel", "Cancel" );
        button.Set_Layout_LTWH( -5, -5, 55, 20 );
        button.Add_Listener( A_Button_Listener(this) );
        this.Add( A_Widget(button) );
    end Construct;

    ----------------------------------------------------------------------------

    not overriding
    procedure Do_Apply( this : access Entity_Attributes_Dialog ) is
        view : constant A_Game_View := A_Game_View(this.Get_View);

        procedure Apply_Attribute( name : String; value : A_Value ) is
            input : constant A_Input_Box := A_Input_Box(view.Get_Widget( To_String( this.id & ":ib" & name ) ));
            newVal: A_Value;
        begin
            if value.As_String /= input.Get_Text then
                newVal := Create_Value( input.Get_Text );
                Queue_Set_Entity_Attribute( this.eid, name, newVal );
            end if;
        end Apply_Attribute;

    begin
        this.eAttributes.Iterate( Apply_Attribute'Access );
        this.Hide;
    end Do_Apply;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this   : access Entity_Attributes_Dialog;
                             action : A_Button_Action ) is
    begin
        if action.Get_Id = Click then
            if action.Get_Source.Get_Id = To_String( this.id & ":btnApply" ) then
                this.Do_Apply;
            elsif action.Get_Source.Get_Id = To_String( this.id & ":btnCancel" ) then
                this.Hide;
            end if;
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this    : access Entity_Attributes_Dialog;
                             action  : A_Key_Action;
                             handled : out Boolean ) is
    begin
        if action.Get_Key = KEY_ESC and then action.Get_Id = Key_Press then
            this.Hide;
            handled := True;
        else
            Dialog(this.all).Handle_Action( action, handled );
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Show( this : access Entity_Attributes_Dialog ) is
    begin
        Dialog(this.all).Show;
        this.Get_Window.Give_Focus( this.Get_View.Get_Widget( this.id & ":btnApply" ) );
    end Show;

end Widgets.Containers.Panels.Dialogs.Entity_Attributes;
