
with Allegro.Keyboard;                  use Allegro.Keyboard;
with Events.Audio;                      use Events.Audio;
with Events.World;                      use Events.World;
with Game_Views;                        use Game_Views;
with Support;                           use Support;
with Widgets.Buttons.Pushes;            use Widgets.Buttons.Pushes;
with Widgets.Buttons.Toggles;           use Widgets.Buttons.Toggles;
with Widgets.Containers.Panels;         use Widgets.Containers.Panels;
with Widgets.Input_Boxes;               use Widgets.Input_Boxes;
with Widgets.Input_Boxes.Constraints;   use Widgets.Input_Boxes.Constraints;
with Widgets.Labels;                    use Widgets.Labels;

package body Widgets.Containers.Panels.Dialogs.Properties is

    function Create_Properties_Dialog( view : not null access Game_Views.Game_View'Class;
                                       id   : String ) return A_Dialog is
        this : A_Dialog := new Properties_Dialog;
    begin
        this.Construct( view, id );
        return this;
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Properties_Dialog;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Properties_Dialog;
                         view : not null access Game_Views.Game_View'Class;
                         id   : String ) is
        button   : A_Button;
        inputbox : A_Input_Box;
        label    : A_Label;
    begin
        Dialog(this.all).Construct( view, id, "World Properties", "" );
        this.Set_Layout_Center( 275, 121 );

        label := Create_Label( view, this.id & ":lblDomain", "Domain:" );
        label.Set_Layout_LTRB( 5, 5, -70, -23 );
        label.Set_Align( Align_Right );
        this.Add( A_Widget(label) );

        label := Create_Label( view, this.id & ":lblDomainVal", "" );
        label.Set_Layout_LTRB( 70, 5, 5, -23 );
        this.Add( A_Widget(label) );

        label := Create_Label( view, this.id & ":lblIntro", "Introduction:" );
        label.Set_Layout_LTRB( 5, 26, -70, -44 );
        label.Set_Align( Align_Right );
        this.Add( A_Widget(label) );

        inputbox := Create_Input_Box( view, this.id & ":ibIntro" );
        inputbox.Set_Layout_LTRB( 70, 26, 5, -44 );
        inputbox.Set_Max_Length( 256 );
        inputbox.Set_Prev( this.id & ":btnCancel" );
        inputbox.Set_Next( this.id & ":ibMusic" );
        this.Add( A_Widget(inputbox) );

        label := Create_Label( view, this.id & ":lblMusic", "Music:" );
        label.Set_Layout_LTRB( 5, 47, -70, -65 );
        label.Set_Align( Align_Right );
        this.Add( A_Widget(label) );

        inputbox := Create_Input_Box( view, this.id & ":ibMusic" );
        inputbox.Set_Layout_LTRB( 70, 47, 26, -65 );
        inputbox.Set_Constraint( Constraints.Basename_Only'Access );
        inputbox.Set_Prev( this.id & ":ibIntro" );
        inputbox.Set_Next( this.id & ":btnMusicPlay" );
        this.Add( A_Widget(inputbox) );

        button := Create_Toggle_Button( view, this.id & ":btnMusicPlay", icon => "play_icon" );
        button.Set_Layout_LTRB( -23, 47, 5, -65 );
        button.Add_Listener( A_Button_Listener(this) );
        button.Set_Prev( this.id & ":ibMusic" );
        button.Set_Next( this.id & ":btnApply" );
        this.Add( A_Widget(button) );

        button := Create_Push_Button( view, this.id & ":btnApply", "Apply" );
        button.Set_Layout_LTWH( -63, -5, 55, 20 );
        button.Add_Listener( A_Button_Listener(this) );
        button.Set_Prev( this.id & ":btnMusicPlay" );
        button.Set_Next( this.id & ":btnCancel" );
        this.Add( A_Widget(button) );

        button := Create_Push_Button( view, this.id & ":btnCancel", "Cancel" );
        button.Set_Layout_LTWH( -5, -5, 55, 20 );
        button.Add_Listener( A_Button_Listener(this) );
        button.Set_Prev( this.id & ":btnApply" );
        button.Set_Next( this.id & ":ibIntro" );
        this.Add( A_Widget(button) );

        this.Get_View.Get_Corral.Add_Listener( A_Event_Listener(this), "World_Property_Changed" );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Properties_Dialog ) is
    begin
        this.Get_View.Get_Corral.Remove_Listener( A_Event_Listener(this), "World_Property_Changed" );
        Dialog(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    not overriding
    procedure Do_Apply( this : access Properties_Dialog ) is
    begin
        Queue_Set_World_Property( "introduction", A_Input_Box(this.Get_View.Get_Widget( this.id & ":ibIntro" )).Get_Text );
        Queue_Set_World_Property( "music", A_Input_Box(this.Get_View.Get_Widget( this.id & ":ibMusic" )).Get_Text );
        this.Hide;
    end Do_Apply;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this   : access Properties_Dialog;
                             action : A_Button_Action ) is
    begin
        if action.Get_Id = Click then
            if action.Get_Source.Get_Id = To_String( this.id & ":btnApply" ) then
                this.Do_Apply;
            elsif action.Get_Source.Get_Id = To_String( this.id & ":btnCancel" ) then
                this.Hide;
            end if;
        elsif action.Get_Id = Press then
            if action.Get_Source.Get_Id = To_String( this.id & ":btnMusicPlay" ) then
                if A_Input_Box(this.Get_View.Get_Widget( this.id & ":ibMusic" )).Get_Text'Length > 0 then
                    A_Button(action.Get_Source).Set_Icon( "pause_icon" );
                    Queue_Play_Music( A_Input_Box(this.Get_View.Get_Widget( this.id & ":ibMusic" )).Get_Text );
                else
                    A_Button(action.Get_Source).Set_State( False );
                end if;
            end if;
        elsif action.Get_Id = Release then
            if action.Get_Source.Get_Id = To_String( this.id & ":btnMusicPlay" ) then
                A_Button(action.Get_Source).Set_Icon( "play_icon" );
                Queue_Stop_Music;
            end if;
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this    : access Properties_Dialog;
                             action  : A_Key_Action;
                             handled : out Boolean ) is
    begin
        if action.Get_Key = KEY_ESC and then action.Get_Id = Key_Press then
            this.Hide;
            handled := True;
        else
            Dialog(this.all).Handle_Action( action, handled );
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Event( this : access Properties_Dialog;
                            evt  : in out A_Event;
                            resp : out Response_Type ) is
        pragma Unreferenced( resp );

        procedure Handle_World_Property_Changed( evt : A_World_Property_Changed_Event ) is
            name : constant String := evt.Get_Property_Name;
            view : constant A_Game_View := A_Game_View(this.Get_View);
        begin
            if name = "music" then
                A_Input_Box(view.Get_Widget( this.id & ":ibMusic" )).Set_Text( evt.Get_Value );
            elsif name = "domain" then
                A_Label(view.Get_Widget( this.id & ":lblDomainVal" )).Set_Text( evt.Get_Value );
            elsif name = "introduction" then
                A_Input_Box(view.Get_Widget( this.id & ":ibIntro" )).Set_Text( evt.Get_Value );
            end if;
        end Handle_World_Property_Changed;

    begin
        if evt.Get_Id = WORLD_PROPERTY_CHANGED_ID then
            Handle_World_Property_Changed( A_World_Property_Changed_Event(evt) );
        end if;
    end Handle_Event;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Hide( this : access Properties_Dialog ) is
        pragma Unreferenced( this );
    begin
        Queue_Stop_Music;
    end Handle_Hide;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Show( this : access Properties_Dialog ) is
        view : constant A_Game_View := A_Game_View(this.Get_View);
    begin
        A_Button(view.Get_Widget( this.id & ":btnMusicPlay" )).Set_State( False );
    end Handle_Show;

    ----------------------------------------------------------------------------

    overriding
    procedure Show( this : access Properties_Dialog ) is
    begin
        Dialog(this.all).Show;
        this.Get_Window.Give_Focus( this.Get_View.Get_Widget( this.id & ":btnCancel" ) );
    end Show;

end Widgets.Containers.Panels.Dialogs.Properties;
