
--with Debugging;                        use Debugging;
with Themes.Palette;                    use Themes.Palette;

package body Widgets.Palettes is

    overriding
    procedure Construct( this : access Palette;
                         view : not null access Game_Views.Game_View'Class;
                         id   : String ) is
    begin
        Widget(this.all).Construct( view, id );
        this.acceptFocus := False;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Draw_Content( this : access Palette; dc : Drawing_Context ) is
        firstRow    : constant Integer := (this.vy1 / (this.iconHeight + 1)) + 1;
        lastRow     : constant Integer := (this.vy2 / (this.iconHeight + 1)) + 1;
        itemCount   : constant Integer := A_Palette(this).Get_Item_Count;
        selectedRow,
        selectedCol : Integer;
        drawX       : Integer;
        drawY       : Integer := (firstRow - 1) * (this.iconHeight + 1);
        sprite      : A_Bitmap;
        itemborder  : Color_Type;
    begin
        this.Get_Selected( selectedRow, selectedCol );

        -- background color
        Rectfill( dc, this.vx1, this.vy1, this.vx2, this.vy2, this.colors(Foreground) );

        for r in firstRow..lastRow loop
            exit when (r - 1) * this.columns > itemCount;

            drawX := 0;
            for c in 1..this.columns loop
                exit when (r - 1) * this.columns + c > itemCount;

                A_Palette(this).Get_Item_Details( r, c, sprite, itemborder );
                if sprite /= null then
                    Stretch_Sprite( dc, sprite, drawX, drawY, this.iconWidth, this.iconHeight, True );
                end if;
                if itemborder /= Themes.Palette.base_colors(Transparent) then
                    Rect( dc, drawX, drawY,
                          drawX + this.iconWidth - 1,
                          drawY + this.iconHeight - 1, itemborder );
                end if;

                if r = this.highlightRow and then c = this.highlightCol then
                    Rectfill( dc, drawX, drawY,
                              drawX + this.iconWidth - 1,
                              drawY + this.iconHeight - 1, this.colors(Selected),
                              opacity => 176 );
                    Rect( dc, drawX, drawY,
                          drawX + this.iconWidth - 1,
                          drawY + this.iconHeight - 1, this.colors(Selected) );
                elsif r = selectedRow and then c = selectedCol then
                    Rectfill( dc, drawX, drawY,
                              drawX + this.iconWidth - 1,
                              drawY + this.iconHeight - 1, this.colors(Selected_Text),
                              opacity => 176 );
                    Rect( dc, drawX, drawY,
                          drawX + this.iconWidth - 1,
                          drawY + this.iconHeight - 1, this.colors(Selected_Text) );
                end if;

                drawX := drawX + this.iconWidth + 1;
            end loop;

            drawY := drawY + this.iconHeight + 1;
        end loop;
    end Draw_Content;

    ----------------------------------------------------------------------------

    overriding
    function Get_Min_Height( this : access Palette ) return Natural is
    begin
        if this.minHeight > 0 then
            return this.minHeight;
        else
            return (A_Palette(this).Get_Item_Count / this.columns + 1) *
                   (this.iconHeight + 1) - 1;
        end if;
    end Get_Min_Height;

    ----------------------------------------------------------------------------

    overriding
    function Get_Min_Width( this : access Palette ) return Natural is
    begin
        if this.minWidth > 0 then
            return this.minWidth;
        else
            return this.columns * (this.iconWidth + 1) - 1;
        end if;
    end Get_Min_Width;

    ----------------------------------------------------------------------------

    overriding
    function Get_Scroll_Inc_X( this : access Palette ) return Integer is
    begin
        return this.iconWidth / 2;
    end Get_Scroll_Inc_X;

    ----------------------------------------------------------------------------

    overriding
    function Get_Scroll_Inc_Y( this : access Palette ) return Integer is
    begin
        return this.iconHeight / 2;
    end Get_Scroll_Inc_Y;

    ----------------------------------------------------------------------------

    procedure Get_Selected( this : not null access Palette'Class;
                            row,
                            col  : out Natural ) is
    begin
        row := 0;
        col := 0;
        if this.selectedItem > 0 then
            row := ((this.selectedItem - 1) / this.columns) + 1;
            col := ((this.selectedItem - 1) mod this.columns) + 1;
        end if;
    end Get_Selected;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Exit( this : access Palette ) is
    begin
        this.highlightRow := 0;
        this.highlightCol := 0;
        A_Widget(this).Set_Dirty;
    end Handle_Exit;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Mouse_Move( this : access Palette;
                                 evt  : not null A_Mouse_Event ) is
        ex     : constant Integer := evt.Get_X;
        ey     : constant Integer := evt.Get_Y;
        oldRow : constant Integer := this.highlightRow;
        oldCol : constant Integer := this.highlightCol;
    begin
        if ex >= this.vx1 and then ex <= this.vx2 and then
           ey >= this.vy1 and then ey <= this.vy2
        then
            this.highlightRow := (ey / (this.iconHeight + 1)) + 1;
            this.highlightCol := (ex / (this.iconWidth + 1)) + 1;
        else
            this.highlightRow := 0;
            this.highlightCol := 0;
        end if;
        if this.highlightRow /= oldRow or else this.highlightCol /= oldCol then
            A_Widget(this).Set_Dirty;
        end if;
    end Handle_Mouse_Move;

    ----------------------------------------------------------------------------

    not overriding
    procedure Set_Columns( this : access Palette; columns : Natural ) is
    begin
        this.columns := columns;
        A_Widget(this).Set_Dirty;
        A_Palette(this).Pack;
    end Set_Columns;

    ----------------------------------------------------------------------------

    not overriding
    procedure Set_Icon_Size( this : access Palette; width, height : Positive ) is
    begin
        this.iconWidth := width;
        this.iconHeight := height;
        A_Widget(this).Set_Dirty;
        A_Palette(this).Pack;
    end Set_Icon_Size;

end Widgets.Palettes;
