
--with Debugging;                         use Debugging;
with Entities;                          use Entities;
with Entities.Players;                  use Entities.Players;
with Impulses;                          use Impulses;
with Games;                             use Games;

pragma Elaborate_All( Games );

package body Games.Keen is

    FIRST_WORLD : constant String := "shadowlands";

    ----------------------------------------------------------------------------

    function Create_Game return A_Game is
        this : constant A_Game := new Keen_Game;
    begin
        this.Construct;
        return this;
    end Create_Game;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Keen_Game ) is
    begin
        Game(this.all).Construct;
        this.Get_Corral.Add_Listener( A_Event_Listener(this), "Give_Ammo" );
        this.Get_Corral.Add_Listener( A_Event_Listener(this), "Give_Drops" );
        this.Get_Corral.Add_Listener( A_Event_Listener(this), "Give_Points" );
        this.Get_Corral.Add_Listener( A_Event_Listener(this), "Impulse" );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Keen_Game ) is
    begin
        this.Get_Corral.Remove_Listener( A_Event_Listener(this), "Impulse" );
        this.Get_Corral.Remove_Listener( A_Event_Listener(this), "Give_Points" );
        this.Get_Corral.Remove_Listener( A_Event_Listener(this), "Give_Drops" );
        this.Get_Corral.Remove_Listener( A_Event_Listener(this), "Give_Ammo" );
        Game(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Event( this : access Keen_Game;
                            evt  : in out A_Event;
                            resp : out Response_Type ) is
    begin
        if evt.Get_Id = IMPULSE_ID then
            this.Handle( A_Impulse_Event(evt) );

        elsif evt.Get_Id = GIVE_POINTS_ID then
            this.Handle( A_Give_Points_Event(evt) );

        elsif evt.Get_Id = GIVE_DROPS_ID then
            this.Handle( A_Give_Drops_Event(evt) );

        elsif evt.Get_Id = GIVE_AMMO_ID then
            this.Handle( A_Give_Ammo_Event(evt) );

        else
            Game(this.all).Handle_Event( evt, resp );
        end if;
    end Handle_Event;

    ----------------------------------------------------------------------------

    not overriding
    procedure Handle( this : access Keen_Game;
                      evt  : not null A_Give_Ammo_Event ) is
    begin
        this.Game_Var_Add( "ammo", evt.Get_Amount );
        if this.sessionVars.As_Integer( "ammo" ) < 0 then
            this.Set_Game_Var( "ammo", 0 );
        end if;
    end Handle;

    ----------------------------------------------------------------------------

    not overriding
    procedure Handle( this : access Keen_Game;
                      evt  : not null A_Give_Drops_Event ) is
    begin
        this.Game_Var_Add( "drops", evt.Get_Amount );
    end Handle;

    ----------------------------------------------------------------------------

    not overriding
    procedure Handle( this : access Keen_Game;
                      evt  : not null A_Give_Points_Event ) is
    begin
        this.Game_Var_Add( "points", evt.Get_Amount );
    end Handle;

    ----------------------------------------------------------------------------

    not overriding
    procedure Handle( this : access Keen_Game;
                      evt  : not null A_Impulse_Event ) is
        e : A_Entity;
    begin
        if not this.paused then
            e := this.world.Get_Entity( evt.Get_Id );
            if e /= null and then e.all in Player'Class then
                A_Player(e).Impulse( evt.Get_Impulse_Name );
            end if;
        end if;
    end Handle;

    ----------------------------------------------------------------------------

    overriding
    procedure New_Game( this : access Keen_Game ) is
    begin
        Game(this.all).New_Game;

        -- setup initial game state
        this.Set_Game_Var( "ammo",     5 );
        this.Set_Game_Var( "drops",    0 );
        this.Set_Game_Var( "points",   0 );
        this.Set_Game_Var( "nextLife", 20000 );
        this.Set_Game_Var( "lives",    3 );
        this.Set_Game_Var( "ancients", 0 );
        this.Set_Game_Var( "scuba",    False );

        -- levels that have not yet been beaten
        this.Set_Game_Var( "level.border", False );

        this.Load_World( FIRST_WORLD );
    end New_Game;

    ----------------------------------------------------------------------------

    overriding
    procedure Pause( this : access Keen_Game; enabled : Boolean ) is
        player : A_Player;
    begin
        Game(this.all).Pause( enabled );
        if enabled then
            -- stop any on-going impulses that keen may be acting on
            player := this.world.Get_Player;
            player.Impulse( Move_Left_Stop );
            player.Impulse( Move_Right_Stop );
            player.Impulse( Move_Up_Stop );
            player.Impulse( Move_Down_Stop );
            player.Impulse( Jump_Stop );
        end if;
    end Pause;

begin

    Register_Allocator( Create_Game'Access );

end Games.Keen;
