
--with Debugging;                         use Debugging;
with Support.Paths;                     use Support.Paths;
with Themes;                            use Themes;

pragma Elaborate_All( Themes );

package body Themes.Keen is

    LIB_NAME : constant String := "interface";

    ----------------------------------------------------------------------------

    function Create_Theme return A_Theme is
        this : A_Theme := new Keen_Theme;
    begin
        this.Construct;
        return this;
    exception
        when others =>
            Delete( this );
            raise;
    end Create_Theme;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Keen_Theme ) is
    begin
        Theme(this.all).Construct;
        this.colors(Background)    := Makeacol( 148, 145, 140, 255 );
        this.colors(Foreground)    := Makeacol( 212, 208, 200, 255 );
        this.colors(Selected)      := Makeacol(  58,  95, 205, 255 );
        this.colors(Selected_Text) := Makeacol( 255, 255, 255, 255 );
        this.colors(Text)          := Makeacol(   0,   0,   0, 255 );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Keen_Theme ) is
    begin
        Unload_Library( this.lib );
        Theme(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    overriding
    function Get_Bitmap( this : access Keen_Theme;
                         id   : Natural ) return A_Bitmap is
    begin
        if this.lib = null then
            this.lib := Load_Library( LIB_NAME );
        end if;
        if this.lib /= null then
            return this.lib.Get_Bitmap_Sync( id );
        end if;
        return null;
    end Get_Bitmap;

    ----------------------------------------------------------------------------

    overriding
    function Get_ID( this : access Keen_Theme; name : String ) return Natural is
        id : Natural;
    begin
        if this.lib = null then
            this.lib := Load_Library( LIB_NAME );
        end if;
        if this.lib /= null then
            id := this.lib.Get_Id( name );
            if id = 0 then
                id := this.lib.Get_Id( Add_Extension( name, "png" ) );
                if id = 0 then
                    id := this.lib.Get_Id( Add_Extension( name, "bmp" ) );
                end if;
            end if;
            return id;
        else
            return 0;
        end if;
    end Get_ID;

begin

    Register_Allocator( Create_Theme'Access );

end Themes.Keen;
