
with Themes.Palette;                    use Themes.Palette;
with Widgets.Containers; use Widgets.Containers;
with Widgets.Layouts;                   use Widgets.Layouts;

package body Widgets.Containers.Keen_Panels is

    procedure Make_Child_Layout( layout      : in out A_Layout;
                                 border      : Border_Type;
                                 borderWidth : Natural ) is
        left,
        top,
        right,
        bottom : Integer := 0;
    begin
        if border /= Border_None then
            left := borderWidth;
            top := borderWidth;
            right := borderWidth;
            bottom := borderWidth;
        end if;
        Delete( layout );
        layout := Create_Layout_LTRB( left, top, right, bottom );
    end Make_Child_Layout;

    ----------------------------------------------------------------------------

    function Create_Panel( view    : not null access Game_Views.Game_View'Class;
                           id      : String;
                           libName : String ) return A_Keen_Panel is
        this : A_Keen_Panel := new Keen_Panel;
    begin
        this.Construct( view, id, libName );
        return this;
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Panel;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this    : access Keen_Panel;
                         view    : not null access Game_Views.Game_View'Class;
                         id      : String;
                         libName : String ) is
    begin
        Container(this.all).Construct( view, id );
        this.transparent := True;
        this.border := Border_Raised;
        this.colors(Background) := base_colors(White);
        this.lib := Load_Library( libName );
        if this.lib /= null then
            this.topId         := this.lib.Get_Id( "dialog-top.png" );
            this.bottomId      := this.lib.Get_Id( "dialog-bottom.png" );
            this.leftId        := this.lib.Get_Id( "dialog-left.png" );
            this.rightId       := this.lib.Get_Id( "dialog-right.png" );
            this.topLeftId     := this.lib.Get_Id( "dialog-topleft.png" );
            this.topRightId    := this.lib.Get_Id( "dialog-topright.png" );
            this.bottomLeftId  := this.lib.Get_Id( "dialog-bottomleft.png" );
            this.bottomRightId := this.lib.Get_Id( "dialog-bottomright.png" );
        end if;
        Make_Child_Layout( this.childLayout, this.border, this.borderWidth );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Keen_Panel ) is
    begin
        Unload_Library( this.lib );
        Container(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    overriding
    procedure Draw_Content( this : access Keen_Panel; dc : Drawing_Context ) is
        bmp : A_Bitmap;
    begin
        Rectfill( dc, this.borderWidth, this.borderWidth,
                  this.cwidth - 1 - this.borderWidth,
                  this.cheight - 1 - this.borderWidth, this.colors(Background) );

        if this.lib /= null then
            -- The corners can have transparency, but the edges can't.
            Draw_Alpha_Sprite( dc, this.lib.Get_Bitmap( this.topLeftId ), 0, 0 );
            Draw_Alpha_Sprite( dc, this.lib.Get_Bitmap( this.topRightId ),
                               this.cwidth - this.borderWidth, 0 );
            Draw_Alpha_Sprite( dc, this.lib.Get_Bitmap( this.bottomLeftId ),
                               0, this.cheight - this.borderWidth );
            Draw_Alpha_Sprite( dc, this.lib.Get_Bitmap( this.bottomRightId ),
                               this.cwidth - this.borderWidth,
                               this.cheight - this.borderWidth );

            if this.cwidth > this.borderWidth * 2 then
                -- The edge bitmaps are stretched to the length of the edge.
                bmp := this.lib.Get_Bitmap( this.topId );
                if bmp /= null then
                    Stretch_Blit( dc,
                                  bmp, 0, 0, Get_Width( bmp ), Get_Height( bmp ),
                                  this.borderWidth, 0,
                                  this.cwidth - this.borderWidth * 2, this.borderWidth );
                end if;
                bmp := this.lib.Get_Bitmap( this.bottomId );
                if bmp /= null then
                    Stretch_Blit( dc,
                                  bmp, 0, 0, Get_Width( bmp ), Get_Height( bmp ),
                                  this.borderWidth, this.cheight - this.borderWidth,
                                  this.cwidth - this.borderWidth * 2, this.borderWidth );
                end if;
                bmp := this.lib.Get_Bitmap( this.leftId );
                if bmp /= null then
                    Stretch_Blit( dc,
                                  bmp, 0, 0, Get_Width( bmp ), Get_Height( bmp ),
                                  0, this.borderWidth,
                                  this.borderWidth, this.cheight - this.borderWidth * 2 );
                end if;
                bmp := this.lib.Get_Bitmap( this.rightId );
                if bmp /= null then
                    Stretch_Blit( dc,
                                  bmp, 0, 0, Get_Width( bmp ), Get_Height( bmp ),
                                  this.cwidth - this.borderWidth, this.borderWidth,
                                  this.borderWidth,  this.cheight - this.borderWidth * 2 );
                end if;
            end if;
        end if;
    end Draw_Content;

    ----------------------------------------------------------------------------

    overriding
    function Get_Min_Height( this : access Keen_Panel ) return Natural is
    begin
        if this.minHeight > 0 then
            return this.minHeight;
        else
            return 2 * this.borderWidth;
        end if;
    end Get_Min_Height;

    ----------------------------------------------------------------------------

    overriding
    function Get_Min_Width( this : access Keen_Panel ) return Natural is
    begin
        if this.minWidth > 0 then
            return this.minWidth;
        else
            return 2 * this.borderWidth;
        end if;
    end Get_Min_Width;

end Widgets.Containers.Keen_Panels;
