
package Entities.Players.Keen is

    type Keen is new Player with private;

    -- Checks if Keen can teleport and then queues the events needed to enter
    -- the teleport state.
    procedure Teleport( this         : access Keen;
                        fromX, fromY : Float;
                        toX,   toY   : Float );

private

    type Action_Type is (Act_Moving, Act_Looking, Act_Looking_Back,
                         Act_Teleporting, Act_Idle, Act_Rising, Act_Dying);

    type Keen is new Player with
        record
            -- *** these fields are streamed ***

            -- true when keen is jumping (actively accelerating upward)
            jumping : Boolean := False;

            -- the world y location of keen when he started his jump. this is
            -- used to determine when the jump has reached min and max heights.
            jumpStartY : Float := 0.0;

            -- true if keen is on his pogo stick. note that jumping and grounded
            -- are independent states and may or may not be true.
            onPogo : Boolean := False;

            -- true if keen is currently shooting. keen may shoot while
            -- performing certain actions. (ie: moving, hanging from a pole)
            shooting : Boolean := False;

            -- the time when keen last fired a shot, used for keeping a delay
            -- between shots. (relative to .age)
            shootStart : Time_Span := Time_Span_Zero;

            -- related to keen performing a one-shot action
            action         : Action_Type := Act_Moving;
            actionStart    : Time_Span := Time_Span_Zero;  -- (relative to .age)
            actionDuration : Time_Span := Time_Span_Zero;

            idleStart  : Time_Span := Time_Span_Zero;      -- (relative to .age)
            idleNumber : Positive := 1;     -- number of the next idle animation

            -- for the teleporting action
            teleportX,                                   -- teleport destination
            teleportY    : Float := 0.0;                 --
            lastTeleport : Time_Span := Time_Span_Zero;  -- time of previous teleport (relative to .age)

            deathX,
            deathY : Float := 0.0;      -- location of keen when he died

            -- ** these fields do not need to be streamed **

            dead : Boolean := False;    -- dead fully dead? (temporary)

            upTime : Time_Span := Time_Span_Zero;

            attemptingJump : Boolean := False;  -- true when the jump key is being held down
            activating     : Boolean := False;  -- true if activate key being held

            minJumpHeight,                  -- entity.player.keen.min_jump_height
            maxJumpHeight,                  -- entity.player.keen.max_jump_height
            minPogoHeight,                  -- entity.player.keen.min_pogo_height
            maxPogoHeight,                  -- entity.player.keen.max_pogo_height
            maxBounceHeight : Float := 0.0; -- entity.player.keen.max_bounce_height
            lookSpeed       : Float := 0.0; -- entity.player.keen.look_speed
        end record;

    procedure Construct( this : access Keen );

    procedure Die( this : access Keen );

    procedure Face( this : access Keen; dir : Direction_Type );

    procedure Impulse( this : access Keen; name : Hashed_String );

    function Is_Busy( this : not null access Keen'Class ) return Boolean;

    procedure Move_Start( this : access Keen; dir : Cardinal_Direction );

    procedure Move_Stop( this : access Keen; dir : Cardinal_Direction );

    function Object_Input( stream : access Root_Stream_Type'Class ) return Keen;
    for Keen'Input use Object_Input;

    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Keen );
    for Keen'Read use Object_Read;

    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Keen );
    for Keen'Write use Object_Write;

    procedure On_Hit_Wall( this : access Keen; dir : Cardinal_Direction );

    procedure Reset_Idle( this : not null access Keen'Class );

    procedure Shoot_Gun( this : not null access Keen'Class );

    procedure Start_Action( this   : not null access Keen'Class;
                            action : Action_Type;
                            length : Time_Span := Time_Span_Zero );

    procedure Start_Jump( this : not null access Keen'Class );

    procedure Stop_Jump( this : not null access Keen'Class );

    procedure Tick( this : access Keen; upTime, dt : Time_Span );

    procedure Update_Frame( this : access Keen; notify : Boolean := True );

end Entities.Players.Keen;
